#include "precompile.h"

#include <a8/mutable_xobject.h>

#include <f8/udplog.h>
#include <f8/msgqueue.h>
#include <f8/jsonhttprequest.h>

#include "handlermgr.h"

#include "GGListener.h"
#include "app.h"
#include "cs_proto.pb.h"
#include "ss_proto.pb.h"

static void _GMOpsSelfChecking(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("healthy", 1);
    request->resp_xobj->SetVal("servicing", App::Instance()->servicing ? 1 : 0);
    #if 0
    request->resp_xobj->SetVal("max_rundelay", PerfMonitor::Instance()->max_run_delay_time);
    request->resp_xobj->SetVal("max_timer_idle", PerfMonitor::Instance()->max_timer_idle);
    #endif
}

static void _GMOpsReload(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    #if 0
    JsonDataMgr::Instance()->Reload();
    f8::UdpLog::Instance()->Info("reload config files", {});
    #endif
}

static void _GMOpsStopService(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    App::Instance()->servicing = false;
}

static void _GMOpsStartService(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    App::Instance()->servicing = true;
}

static void _GMOpsServerInfo(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("node_id", App::Instance()->node_id);
    request->resp_xobj->SetVal("instance_id", App::Instance()->instance_id);
}

static void _GMStatGetRealTimeOnline(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
#if 0
    request->resp_xobj->SetVal("num", PerfMonitor::Instance()->entity_num[ET_Player]);
    request->resp_xobj->SetVal("room_num", RoomMgr::Instance()->RoomNum());
#endif
}

void HandlerMgr::Init()
{
    RegisterNetMsgHandlers();
    RegisterGMMsgHandler("Ops@selfChecking", _GMOpsSelfChecking);
    RegisterGMMsgHandler("Ops@reload", _GMOpsReload);
    RegisterGMMsgHandler("Ops@stopService", _GMOpsStopService);
    RegisterGMMsgHandler("Ops@startService", _GMOpsStartService);
    RegisterGMMsgHandler("Ops@serverInfo", _GMOpsServerInfo);
    RegisterGMMsgHandler("Stat@getRealTimeOnline", _GMStatGetRealTimeOnline);
    f8::MsgQueue::Instance()->RegisterCallBack
        (
         IM_ExecGM,
         [] (const a8::Args& args)
         {
             int socket_handle = args.Get<int>(0);
             std::string url = args.Get<std::string>(1);
             std::string query_str = args.Get<std::string>(2);
             unsigned long saddr = args.Get<unsigned long>(3);

             HandlerMgr::Instance()->ProcGMMsg
                 (
                  saddr,
                  socket_handle,
                  url,
                  query_str
                  );
         });
}

void HandlerMgr::UnInit()
{
}

void HandlerMgr::RegisterNetMsgHandlers()
{

}

void HandlerMgr::ProcGMMsg(unsigned long saddr, int sockhandle,
                           const std::string& url, const std::string& querystr)
{
    if (url != "/webapp/index.php") {
        GGListener::Instance()->SendText(sockhandle, a8::HttpResponse(404, ""));
        return;
    }

    a8::HTTPRequest request;
    a8::ParserUrlQueryString(querystr.c_str(), request);

    std::string msgname = a8::Get(request, "c").GetString() + "@" + a8::Get(request, "a").GetString();
    auto itr = gmhandlers_.find(msgname);
    if (itr != gmhandlers_.end()) {
        auto request = std::make_shared<f8::JsonHttpRequest>();
        request->saddr = saddr;
        request->socket_handle = sockhandle;
        request->query_str = querystr;
        request->params->ReadFromUrlQueryString(querystr);
        request->resp_xobj->SetVal("errcode", 0);
        request->resp_xobj->SetVal("errmsg", "");
        itr->second(request);

        if (!request->pending){
            std::string response;
            request->resp_xobj->ToJsonStr(response);
            GGListener::Instance()->SendText(sockhandle, a8::HttpResponse(response));
        }
    } else {
        GGListener::Instance()->SendText(sockhandle, a8::HttpResponse("{}"));
    }
}

void HandlerMgr::RegisterGMMsgHandler(const std::string& msgname,
                                      void (*handler)(std::shared_ptr<f8::JsonHttpRequest>))
{
    gmhandlers_[msgname] = handler;
}
